@@ -289,7 +289,7 @@ typedef void (*custom_draw_callback)(Fl_Widget *, void *);
         delete ((widget##_Derived *)self);                                                         \
     }                                                                                              \
     void widget##_set_image(widget *self, void *image) {                                           \
-        LOCK(self->image(((Fl_Image *)image)))                                                     \
+        LOCK(auto old = self->image(); self->image(((Fl_Image *)image)->copy()); delete old;)      \
     }                                                                                              \
     void widget##_handle(widget *self, custom_handler_callback cb, void *data) {                   \
         LOCK(((widget##_Derived *)self)->set_handler_data(data);                                   \
@@ -302,7 +302,7 @@ typedef void (*custom_draw_callback)(Fl_Widget *, void *);
         return self->when();                                                                       \
     }                                                                                              \
     void *widget##_image(const widget *self) {                                                     \
-        return (Fl_Image *)self->image();                                                          \
+        return ((Fl_Image *)self->image())->copy();                                                \
     }                                                                                              \
     void widget##_draw(widget *self, custom_draw_callback cb, void *data) {                        \
         LOCK(((widget##_Derived *)self)->set_drawer_data(data);                                    \
@@ -383,10 +383,10 @@ typedef void (*custom_draw_callback)(Fl_Widget *, void *);
         return self->as_group();                                                                   \
     }                                                                                              \
     void widget##_set_deimage(widget *self, void *image) {                                         \
-        LOCK(self->deimage(((Fl_Image *)image)))                                                   \
+        LOCK(auto old = self->image(); self->deimage(((Fl_Image *)image)->copy()); delete old;)    \
     }                                                                                              \
     void *widget##_deimage(const widget *self) {                                                   \
-        return (Fl_Image *)self->deimage();                                                        \
+        return ((Fl_Image *)self->deimage())->copy();                                              \
     }                                                                                              \
     void widget##_set_callback(widget *self, Fl_Callback *cb, void *data) {                        \
         LOCK(self->callback(cb, data);)                                                            \
